/**********************************************************************************************************************************
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * Task-Oriented Quick Tracking and Planning feature for Rational Team Concert and Jazzhub.
 * (c) Copyright IBM Corporation 2014, 2015.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
 **********************************************************************************************************************************/

/**
 * Most work item collection views are divided into 'sections', where each
 * section typically defines an url that must be made to the server.
 */
define(['jquery', 'backbone', 'framework/App', 
		'areas/filters/models/Filters',
		'areas/filters/models/FilterConstants',
		'areas/workitems/models/WorkItemConstants',
		'areas/projects/models/MemberCollection',
		'areas/workitems/models/GroupCollection',
		"areas/workitems/models/WorkItemCollection"
		], 
function($, Backbone, App, Filters, FilterConstants, Constants, MemberCollection, GroupCollection, WorkItemCollection) {
	
	var addUrlParams= function(params) {
		var urlParams = $.deparam(window.location.search.substring(1));
		
		if (urlParams.limit) {
			params.limit= urlParams.limit;
		}
		
		if (urlParams.pageSize) {
			params.pageSize= urlParams.pageSize;
		}
		
		return params;
	};
	
	return Backbone.Model.extend({
		
		// section properties
		
		/**
		 * We don't want to kick off another ajax request while we are already making one.
		 * Because ajax requests are asynchronous, making multiple ajax requests introduces
		 * unpredictability in rendering results (e.g. the latest one can come back first and
		 * the older one can come back second, making the view be out of sync). 
		 */
		refreshing: false,
		
		/**
		 * Because we only handle one request at a time, we need a mechanism to make sure we
		 * don't ignore users request to update the results. For example, if we the ajax request
		 * is in-flight, and the user changes the filters, then we want to make sure we queue the 
		 * new request and process it once the current ajax request completes.
		 */
		queueAnotherRefresh: false,
		
		collection: null, // Collection model
		currentFilters: [],

		defaults: {
			/**
			 * The functional ID is a hard-coded ID assigned to each
			 * section to identify it in code and URLs. Give each
			 * unique section a specific hardcoded ID and stick with it.
			 */
			functionalId: "",
			
			label: null,
			icon: null,
			
			/**
			 * Options for work item creation. Possible values include
			 * noCreate (hides quick create textbox), stateGroupId
			 * (makes second request to move work item to state group
			 * if possible after creation)
			 */
			createOptions: {},
			
			/**
			 * If true, then displays the sprint filter.
			 */
			showSprintFilter: false,
			
			/**
			 * Specifies the ordering of groups. Override this method
			 * to return a group label that would group the given item
			 * with other similar items.
			 */
			getGroup: function(item) {
				return "";
			}
		},
		
		initialize: function(options) {
			options= options || {};
			this.parentSection= options.parentSection || null;
			this.groupCollection = new GroupCollection();

		},
		
		createCollection: function(context) {
			this.collection= new WorkItemCollection([], context, this);
			this.trigger("collectionCreated", this.collection);
		},
		
		getContext: function() {
			return this.get('context');
		},
		
		fetch: function(filters) {
			this.currentFilters= _.clone(filters || []);
			if (!this.refreshing) {
				this.refreshing= true;
				this.collection.reset();
				if (this.get('noResults')) { 
					// TODO: Provide a comment about this if clause
					this.collection.trigger('sync');
				} else {
					this.doFetch(this.currentFilters);
				}	
			} else {
				this.queueAnotherRefresh = true;
			}
		},
		
		doFetch: function(filters) {
			var self= this;
			var context= this.getContext();
			var fetch = function(){
				var params = addUrlParams(self.getRequestParams(filters));
				var query= self.toQuery(params);
				App.controller.filterController.setCurrentlyShowingQuery(query);	
				self.collection.fetch({
					url: self.getRequestUrl(),
					type: self.getRequestType(),
					data: $.param(params, true), // true is for traditional parameterization for arrays
					reset: true,
					success: function() {
						// the context here is whatever supplied by jquery to success
						// we need to make sure the 'this' parameter is AbstractSection
						self._fetchCompleted.call(self);
					},
					error: function(model, resp, options) {
						// the context here is whatever supplied by jquery to error
						// we need to make sure the 'this' parameter is AbstractSection
						self._fetchError.call(self, model, resp, options);
					}
				});
			};
			if (App.isJazzhubProject(context.project)) {
				// In Jazzhub, we're going to want the project's members to be
				// cached before the work items are displayed, for efficient loading
				// user user photos.
				MemberCollection.withMembers(context).done(fetch);
			} else {
				fetch();
			}
		},

		_checkQueuedRefresh: function() {
			if (this.queueAnotherRefresh) {
				this.queueAnotherRefresh = false;
				
				// use the currentFilters. Keep in mind we always 
				// store the current filters as part of the fetch() call.
				this.fetch(this.currentFilters);
			}	
		},
		
		_fetchCompleted: function() {
			this.refreshing = false;
			this._checkQueuedRefresh();
		},

		_fetchError: function(model, resp, options) {
			this.refreshing = false;
			
			// don't hide the spinnner and flash an error if we just have
			// to make another request anyway.
			if (!this.queueAnotherRefresh) {
				this.collection.trigger('error', model, resp, options);
			}
			
			this._checkQueuedRefresh();
		},
		
		getRequestUrl: function() {
			throw new Error("Should be implemented by sub classes");
		},
		
		getRequestParams: function(filters) {
			return Filters.toParams(filters);
		},
		
		toQuery: function(params) {
			var filters= this._getFiltersFromParams(params),
				json={ 
						operator: "AND", 
						attributeExpressions: _.map(filters, Filters.toQueryExpression)
					  };
						 
			return {
					startIndex: 0,
					columnIdentifiers: params.columns,
					sortColumns: [params.sortAsc, params.sortDsc],
					sortDirections: [true, false],
					projectAreaItemId: params.projectArea,
					jsonExpression: JSON.stringify(json) 
				};
		},
		
		_getFiltersFromParams: function(params) {
			var filters= [];
			if (params.iteration) {
				filters.push(Filters.toFilter([params.iteration], FilterConstants.typeIteration));
			}
			if (params.teamArea) {
				filters.push(Filters.toFilter([params.teamArea], FilterConstants.typeTeamArea));
			}
			if (params.filters) {
				filters= filters.concat(Filters.fromParams($.parseJSON(params.filters)));
			}
			return filters;
		},
		
		getRequestType: function() {
			return "POST";
		},
		
		refreshItem: function(workItems) {
			var filters= _.clone(this.currentFilters).concat(Filters.toFilter(
				_.map(
					workItems,
					function(workItem) {
						return workItem.get(Constants.propNumber).toString();
					}
				),
				FilterConstants.typeId)
			);
			var params= this.getRequestParams(filters);
			if (params.columns.indexOf(Constants.propParent) === -1) {
				params.columns.push(Constants.propParent);
			}
			var options= {
				url: this.getRequestUrl(),
				type: this.getRequestType(),
				data: $.param(params, true)
			};

			return this.collection.refreshItem(workItems, options);
		},

		getRootSection: function() {
			var root= this;
			while (root.getParentSection() !== null) {
				root= root.getParentSection();
			}
			return root;
		},

		getParentItem: function() {
			return this.parentItem || null;
		},
		
		getParentSection: function() {
			return this.parentSection;
		},
		
		_getSelectedTeamAreas: function(filters) {
			var teamAreasFilter= _.filter(filters, function(filter) {
				return filter.key === FilterConstants.typeTeamArea;
			});
			return teamAreasFilter.length > 0 ? teamAreasFilter[0].values : [];
		},
		
		_getCql: function(filters) {
			var cqlFilter= _.filter(filters, function(filter) {
				return filter.key === FilterConstants.typeCql;
			});
			return cqlFilter.length > 0 ? cqlFilter[0].values : [];
		}
	});
});