/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
/**
 * Backlog section
 */
define(['jquery', 'backbone', 'underscore', 'uri',
		'framework/App', 'moment',
		'areas/workitems/models/WorkItemConstants',
        'areas/workitems/models/AbstractSection',
		'areas/filters/models/Filters',
		'areas/filters/models/FilterConstants',
		"areas/workitems/models/RankedWorkItemCollection",
		'areas/servers/models/ServerConstants',
		'areas/iterations/models/IterationConstants'], 
function($, Backbone, _, URI,
		App,
		Moment,
		Constants, 
		AbstractSection,
		Filters,
		FilterConstants,
		RankedWorkItemCollection,
		ServerConstants,
		IterationConstants) {
	
	return AbstractSection.extend({
		
		initialize: function initialize() {
			AbstractSection.prototype.initialize.call(this);
			var iteration = this.get('context').iteration;
				
			this.set('dropFn', _.bind(this._onDrop, this));
			this.set('canDropFn', _.bind(this.canDrop, this));
			this.set('iteration', iteration);
			this.set('functionalId', iteration.get('id'));
			this.set('label', iteration.get(Constants.propLabel) || iteration.get('id'));
			this.set('icon', iteration.getIconClass());
			this.set('secondaryLabel', this._createSecondaryLabel(iteration));
		},
		
		createCollection: function(context) {
			this.collection = new RankedWorkItemCollection([], context, this);
			this.set(FilterConstants.excludeResolved, this.get("initialExcludeResolved"));
		},
		
		/**
		 * Creates a secondary label for the section using iteration dates. For example,
		 * an Iteration named Iteration 1 that starts Nov 1, 2014 and ends Nov 14, 2014
		 * will have a combined primary and secondary label as:
		 * 
		 * Iteration 1 (Nov 1 - Nov 14, 2014)
		 */
		_createSecondaryLabel: function(iteration) {
			var startDate = iteration.get(IterationConstants.propStartDate);
			startDate = startDate ? Moment(startDate, IterationConstants.DateFormat) : null;
			var endDate = iteration.get(IterationConstants.propEndDate);
			endDate = endDate ? Moment(endDate, IterationConstants.DateFormat) : null;
			
			var labelFormat= null;
			var formatArgs= [];
			var sameYear= endDate !== null && startDate !== null && endDate.year() === startDate.year();
			if (startDate !== null) {
				labelFormat= App.messages.SprintPlanTreeSection_iterationWithOnlyStartDateLabel;
				formatArgs.push(startDate.format(sameYear ? App.messages.SprintPlanTreeSection_iterationStartDateSameYearFormat : App.messages.SprintPlanTreeSection_iterationStartDateFormat));
			} 
			if (endDate !== null) {
				labelFormat= App.messages.SprintPlanTreeSection_iterationWithOnlyEndDateLabel;
				formatArgs.push(endDate.format(sameYear ? App.messages.SprintPlanTreeSection_iterationEndDateSameYearFormat : App.messages.SprintPlanTreeSection_iterationEndDateFormat));
			} 
			if (endDate !== null && startDate !== null) {
				labelFormat= App.messages.SprintPlanTreeSection_iterationDateLabel;
			}
			return labelFormat !== null ? labelFormat.format.apply(labelFormat, formatArgs) : null;
		},
		
		_onDrop: function(draggable) {
			var iteration= this.get('iteration');
			var context= this.get('context');
			var model= draggable.model;
			if (model.get(Constants.propIteration).id === iteration.id) {
				return;
			}
			// Following should go into work item model
			App.controller.workItemController.updateWorkItem(context, model, 
				{'target': iteration.id},
				draggable.updateMonitor,
				function() {
					model.set({iteration: iteration});
					if (draggable.success) {
						draggable.success();
					}
				},
				draggable.error
			);
		},
		
		canDrop: function(draggable, callback) {
			var model= draggable.model,
				modelRootSection= model.collection.getSection().getRootSection(),
				modelProject= model.get(Constants.propProject),
				contextProject= this.getContext().project;
			
			if (modelRootSection === this) {
				return;
			}
			
			callback(contextProject.id === modelProject.id);
		},
		
		getRequestUrl: function() {
			return new URI(this.getContext().server.get(ServerConstants.propUrl))
							.segment("qp")
							.segment("data")
							.segment("sprintPlanTree");
		},
		
		getRequestParams: function(filters) {
			var params= {};

			params.pageSize= 50;
			params.columns= this.get('columns');
			params.sortAsc= this.get('sortAscending');
			params.sortDsc= this.get('sortDescending');
			params._method= "GET";
			params.projectArea= this.getContext().project.get(Constants.propItemId);
			params.iteration= this.get('iteration').get('id');
			
			var teamAreas= this._getSelectedTeamAreas(filters);
			if (teamAreas.length > 0) {
				params.teamArea= teamAreas[0];
			}
			
			var cql = this._getCql(filters);
			if (cql.length > 0) {
				params[FilterConstants.typeCql] = cql[0];
			}
			
			var additionalFilters= this._getAdditionalFilters(filters || []);
			if (additionalFilters.length > 0) {
				params.filters= JSON.stringify(Filters.toParams(additionalFilters));
			}
			
			return params;
		},
		
		_getAdditionalFilters: function(filters) {
			var result= _.filter(filters, function(filter) {
				return filter.key !== FilterConstants.typeTeamArea && 
					filter.key !== FilterConstants.typeIteration &&
					filter.key !== FilterConstants.typeCql;
			});
			if (this.get(FilterConstants.excludeResolved) !== false) {
			    result.push(Filters.toFilter(true, FilterConstants.excludeResolved));
			}
			return result;
		}
	});
});
